#!/usr/bin/python2
import json, re
import yarn

def yarn2dot(y, out):
    print >> out, 'digraph {'
    for name in y.passages:
        for dest, typ in findLinks(y.passages[name]):
            e = '%s -> %s' % (json.dumps(name), json.dumps(dest))
            if typ == 'display':
                e += ' [style=dashed]'
            elif typ == 'js':
                e += ' [style=dotted]'
            print >> out, e + ';'
            if typ == 'dispexpr':
                print >> out, '%s [shape=box];' % json.dumps(dest)
    print >> out, '}'

def findLinks(content):
    for line, typ in yarn.parseContent(content):
        for m in re.finditer(r'\[\[(.*?)\]\]', line):
            text = dest = m.group(1)
            if '|' in text:
                dest, text = text.split('|', 1)
            yield dest, typ
        for m in re.finditer(r'''display\s*\(\s*(.*?)\s*\)''', line):
            dest = m.group(1)
            if dest[:1] == dest[-1:] and dest[:1] in ('"', "'"):
                dest = dest[1:-1]
                typ = 'display'
            else:
                typ = 'dispexpr'
            yield dest, typ

if __name__ == '__main__':
    import os, sys, argparse
    
    ap = argparse.ArgumentParser()
    ap.add_argument('yarnfile', nargs='+')
    ap.add_argument('-o', '--out', metavar='pngfile', help='instead of outputting dot script, invoke dot and write the resulting PNG to the named file')
    args = ap.parse_args()

    y = yarn.Yarn()
    for f in args.yarnfile:
        y.parse(open(f, 'r'))
    
    if args.out:
        from subprocess import call
        buf = os.tmpfile()
        yarn2dot(y, buf)
        buf.seek(0)
        if call(['dot', '-Tpng'], stdin=buf, stdout=open(args.out, 'wb')):
            print >> sys.stderr, 'dot failed'
        else:
            print >> sys.stderr, 'wrote PNG to', args.out
    else:
        yarn2dot(y, sys.stdout)
