
if (exports) {
	exports.makeHTML = makeHTML;
	exports.parseHTML = parseHTML;

	htmltemplate = require('./editor_data.js').htmltemplate;
}

function parseHTML(data)
{
	if (typeof(data) == 'string')
		data = data.split(/\n/);
	var yarn = false;
	var p = '';
	for (var i = 0; i < data.length; i++) {
		if (data[i] == 'BEGIN YARN PASSAGES')
			yarn = true;
		else if (yarn) {
			if (data[i] == 'END YARN PASSAGES') {
				passages = JSON.parse(p);
				return true;
			} else {
				p += data[i];
			}
		}
	}
	return false;
}

function makeHTML(extrajs)
{
	if (!extrajs) extrajs = '';
	var html = htmltemplate.replace(/^\$GAME$/m, makeJS() + '\n' + extrajs);
	return html;
}

function makeJS()
{
	var names = Object.keys(passages);
	names.sort();
	var js = '';
	for (var i = 0; i < names.length; i++) {
		if (names[i] == '')
			js += renderContent(passages[names[i]], true);
		else
			js += '\n\npassage[' + JSON.stringify(names[i]) + '] = function() {\n' + renderContent(passages[names[i]]) + '\n};\n';
	}

	js += '/*\nBEGIN YARN PASSAGES\n{';
	var re = new RegExp('\\*/', 'g');
	for (var i = 0; i < names.length; i++) {
		var p = '\n' + JSON.stringify(names[i]) + ':' + JSON.stringify(passages[names[i]]);
		if (i != names.length - 1)
			p += ',';
		// escape the multiline comment closer ('*/') so it doesn't bork the PASSAGES comment
		p = p.replace(re, '*\\u002f');
		js += p;
	}
	js += '\n}\nEND YARN PASSAGES\n*/\n';
	return js;
}

function renderContent(content, jsonly)
{
	content = content.replace(/\s*$/, '');
	return _renderContentSub(content, jsonly).join('\n');
}

function _renderContentSub(content, jsonly)
{
	var lines = [];
	var list = parseContent(content);
	for (var i = 0; i < list.length; i++) {
		switch (list[i].type) {
		case 'text':
			if (!jsonly)
				lines.push('write(' + JSON.stringify(list[i].line) + ');');
			break;
		case 'js':
			lines.push(list[i].line);
			break;
		}
	}
	return lines;
}

function parseContent(content)
{
	var list = [];
	var lines = content.split(/\n/g);
	var js = false;
	for (var i = 0; i < lines.length; i++) {
		var line = lines[i].replace(/\s*$/, '');
		if (line == '${')
			js = true;
		else if (line == '$}')
			js = false;
		else if (js)
			list.push({'type': 'js', 'line': line});
		else if (line.substring(0, 1) == '$')
			list.push({'type': 'js', 'line': line.substring(1)});
		else
			list.push({'type': 'text', 'line': line});
	}
	return list;
}

function findLinks(content)
{
	var lines = parseContent(content);
	var links = [];
	for (var i = 0; i < lines.length; i++) {
		var re;
		var m;

		// regular [[links]]
		re = /\[\[(.*?)\]\]/g;
		while ((m = re.exec(lines[i].line)) != null) {
			var text = m[1];
			var dest = m[1];
			if (text.search(/\|/) != -1) {
				var x = text.split(/\|/, 1);
				dest = x[0];
				text = x[1];
			}
			links.push({'dest': dest, 'type': lines[i].type});
		}

		// display() links (obvious ones, anyway)
		re = /display\s*\(\s*(.*?)\s*\)/g;
		while ((m = re.exec(lines[i].line)) != null) {
			var a = m[1].substring(0, 1);
			var b = m[1].substring(m[1].length - 1);
			if (a == b && (a == '"' || a == "'")) {
				links.push({
					'dest': m[1].substring(1, m[1].length - 1),
					'type': 'display',
				});
			} else {
				links.push({
					'dest': m[1],
					'type': 'dispexpr',
				});
			}
		}
	}
	return links;
}
