#!/usr/bin/env node

var args = process.argv.splice(2);

if (args.length == 0) {
	console.log('Usage: node tool.js COMMAND (ARGS...)\n');
	console.log('    yarn\n        read an html from stdin and produce a yarn file on stdout');
	console.log('    html\n        read a yarn from stdin and produce an html file on stdout');
	process.exit(1);
}

function yarn2html()
{
	var render = require('./editor/editor_render.js');

	passages = {'': ''};

	var name = '';
	for (var i = 0; i < lines.length; i++) {
		if (lines[i].search(/::/) == 0) {
			name = lines[i].substring(2).trim();
			passages[name] = '';
		} else {
			passages[name] += lines[i] + '\n';
		}
	}

	// output html
	process.stdout.write(render.makeHTML());
}

function html2yarn()
{
	var render = require('./editor/editor_render.js');

	if (!render.parseHTML(lines)) {
		console.log('ERROR: No BEGIN/END YARN PASSAGES found in input.');
		process.exit(1);
	}

	// write yarn file
	var names = Object.keys(passages);
	names.sort();
	for (var i = 0; i < names.length; i++) {
		if (names[i] != '')
			process.stdout.write(':: ' + names[i] + '\n');
		process.stdout.write(passages[names[i]].replace(/\s*$/, '') + '\n\n');
	}
}

var func;

switch (args[0]) {
case 'yarn':
	func = html2yarn;
	break;
case 'html':
	func = yarn2html;
	break;

default:
	console.log('Unknown command: ' + args[0]);
	process.exit(1);
	break;
}


var ibuf = '';
var lines = [];

process.stdin.resume();
process.stdin.setEncoding('utf8');

process.stdin.on('data', function(chunk) {
	ibuf += chunk;
	var list = ibuf.split(/\r?\n/);
	ibuf = list.pop();
	lines = lines.concat(list);
});

process.stdin.on('end', function() {
	lines.push(ibuf);
	func();
});
