
var debug = false;  // make true to enable some debug features
var initerror = 0;  // 0 = before loading game, 1 = init error, 2 = game loaded
var loadcookie = true;  // set false to disable savegame loading (useful for debugging)

var info = {
	//title: '',
	//author: '',
	start: 'start',
	restart: true,
	version: 0,
};
var save = {};
var _save = {lines: []};
var passage = {};

var linksilly = false;  // if set true, links are of the form [[display text|destination]] (twine) instead of the normal [[destination|display text]] (wikipedia)

var fadeTick = 1000 / 30;
var fadeLength = .4;

var fadeAmount;
function startFade()
{
	fadeAmount = 0;
	continueFade();
}
function continueFade()
{
	fadeAmount += (fadeTick / 1000) / fadeLength;
	if (fadeAmount > 1) fadeAmount = 1;

	var page = document.getElementById('page');
	page.style.opacity = fadeAmount;
	if (fadeAmount < 1)
		setTimeout(continueFade, fadeTick);
}

function menuClear()
{
	document.getElementById('menu').innerHTML = '';
}
function menuAdd(html, cls)
{
	var menu = document.getElementById('menu');
	var div = document.createElement('div');
	div.className = 'menuitem';
	if (cls) div.className += ' ' + cls;
	div.innerHTML = html;
	menu.appendChild(div);
}

function menuAddRestart()
{
	menuAdd('<a href="#" id="menu_restart">restart</a>');

	document.getElementById('menu_restart').onclick = function() {
		if (debug || confirm('Really erase all progress and start over?'))
			deleteGame();
		return false;
	};
}
function menuUpdateDebug()
{
	var dbg = document.getElementById('debug');
	var html = '';

	html += '<p><form><input id="debug_goto" value="' + _save.passage + '"><input type="submit" value="goto" id="debug_gotobutton"></form>';

	html += '<p>';
	for (var name in save) {
		if (!name.search(/_/) == 0)
			html += name + ' = ' + JSON.stringify(save[name]) + '<br>';
	}

	dbg.innerHTML = html;

	document.getElementById('debug_gotobutton').onclick = function() {
		gotoPassage(document.getElementById('debug_goto').value);
		return false;
	};
}

function getCookieName()
{
	return 'yarnsave_' + escape(info.title) + info.version.toString();
}

function saveGame()
{
	var s = {save: save, _save: _save};
	createCookie(getCookieName(), JSON.stringify(s), 365 * 3);
}

function loadGame()
{
	var data = readCookie(getCookieName());
	if (data) {
		var s = JSON.parse(data);
		save = s.save;
		_save = s._save;
		return true;
	} else return false;
}

function deleteGame()
{
	eraseCookie(getCookieName());
	location.reload(true);
}

function display(name)
{
	var p = passage[name];

	if (p) {
		try {
			p();
		} catch (e) {
			showError(e, name);
		}

	} else {
		write('<b>ERROR</b>: Missing passage "' + name + '"');
	}
}

window.onerror = function(msg, url, line)
{
	if (initerror == 0)
		initerror = 1;
	var e = {
		name: msg.split(/:/)[0],
		msg: msg.split(/:/)[1],
		fileName: url,
		lineNumber: line,
	};
	showError(e);
}

function showError(e, passageName)
{
	write('<b>ERROR</b>: Script failed.');
	if (debug || initerror == 1) {
		if (passageName)
			write('(in passage "' + passageName + '")');
		write(e.name + ': ' + e.msg);
		if (e.fileName.search(/data:/) == 0)
			e.fileName = e.fileName.substring(0, 32) + '...';
		write('line ' + e.lineNumber + ' in ' + e.fileName);
		write();
		write('The following may be offset, but is probably useful:');
		var doclines = document.firstElementChild.outerHTML.split(/\n/);
		for (var x = -2; x <= 2; x++) {
			// -2 = -1 (because line 1 is element 0) -1 (doctype isn't included in html tag's outerhtml)
			var s = (e.lineNumber + x) + ': ' + doclines[e.lineNumber - 2 + x];
			if (x == 0) s = '<b>' + s + '</b>';
			write(s);
		}
	}
}

function startGame()
{
	if (initerror == 1)
		return;
	initerror = 2;

	// build menu
	if (info.title)
		menuAdd(info.title, 'title');
	if (info.author)
		menuAdd('by ' + info.author, 'author');

	if (info.restart)
		menuAddRestart();

	menuAdd('made with <a href="http://raincomplex.net/yarn">yarn</a>', 'madewith');
	
	// load or start new game
	if (loadcookie && loadGame())
		showFromSave();
	else
		gotoPassage(info.start);
}

function showFromSave()
{
	var lines = _save.lines;
	clear();

	write('<small>(loaded from save cookie)</small>\n');
	for (var i = 0; i < lines.length; i++)
		write(lines[i]);

	startFade();
	if (debug) menuUpdateDebug();
}

function gotoPassage(dest)
{
	clear();

	_save.passage = dest;
	display(dest);
	saveGame();

	startFade();
	if (debug) menuUpdateDebug();
}

var linkid = 0;

function clear()
{
	linkid = 0;
	_save.lines = [];

	var page = document.getElementById('page');
	page.style.opacity = 0;
	page.innerHTML = '';
}

function write(line)
{
	// if line is an array, recurse on each element
	if (Array.isArray(line)) {
		for (var i = 0; i < line.length; i++)
			write(line[i]);
		return;
	}

	if (!line)
		line = '';

	if (line.search(/\n/) != -1) {
		write(line.split(/\n/g));
		return;
	}
	
	// store the line in case we need to load this passage
	_save.lines.push(line);

	var page = document.getElementById('page');

	// replace [[link]]s with <a>s
	var makelinks = [];
	function repl(match, text) {
		linkid++;
		var link = {
			id: 'link' + linkid,
			dest: text,
		};
		var pipe = text.search(/\|/);
		if (pipe != -1) {
			if (!linksilly) {
				link.dest = text.substring(0, pipe);
				text = text.substring(pipe + 1);
			} else {
				link.dest = text.substring(pipe + 1);
				text = text.substring(0, pipe);
			}
		}
		makelinks.push(link);
		return '<a href="#" id="' + link.id + '">' + text + '</a>';
	}
	line = line.replace(/\[\[(.*?)\]\]/g, repl);

	// make sure the line takes up at least one line's worth of vertical space
	line += '<br>';

	// add the line to the page
	var div = document.createElement('div');
	div.innerHTML = line;
	page.appendChild(div);

	// add onclicks to <a>s
	for (var i = 0; i < makelinks.length; i++) {
		document.getElementById(makelinks[i].id).onclick = function(link) {
			return function() {
				gotoPassage(link.dest);
				return false;
			}
		}(makelinks[i]);
	}
}

function randint(a, b)
{
	return Math.floor(a + Math.random() * (b - a + 1));
}

function setColors(fg, bg)
{
	document.body.style.color = fg;
	document.body.style.backgroundColor = bg;
}
